function [ out ] = Bground( mean1, mean2, maxnum )

% BGROUND - Creates a Poissonian background input state

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% INPUT
% mean1 - mean of the desired Poissonian distribution in one TMD
% mean2 - mean of the desired Poissonian distribution in the other TMD
% maxnum - maximum number of photons to consider

% OUTPUT
% B - the background matrix

% NOTE: To ensure the transformed state is physical, the mean must be
% chosen to be sufficiently low OR maxnum must be chosen to be sufficiently
% high that the Poissonian distributions are not significantly truncated.

N = maxnum;

b1 = zeros(N+1,N+1);
b2 = zeros(N+1,N+1);
B = zeros(N+1,N+1);

numvals = (0:N)';
poiss1 = poisspdf(numvals, mean1);
poiss2 = poisspdf(numvals, mean2)';

for n = 1:(N+1);
    b1(:,n) = poiss1(1:N+1);
    b2(n,:) = poiss2(1:N+1);
end

B = b1.*b2;

out = B;