function  L = loss(l, N)
% LOSS - Creates a loss matrix, L, from a loss parameter, l, and the
% maximum photon number to be considered, N. e.g. for a 9x9 matrix of
% photon numbers 0-8, put N=8.

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% DEPENDENCIES
% Function 'nck.m' must be available - computes binomial coefficients
% without displaying errors

% INPUT
% l - loss parameter (1 - efficiency of detector)
% N - maximum number of photons to consider

% OUTPUT 
% L - Loss matrix

% -----
% START
% -----

n = 0; % Number of photons incident
m = 0; % m = k+1, where k = number of photons detected

L = zeros(N+1,N+1);

L(1,1)=1;
for n = 1:N
    for m = 1:(n+1)
        L(m,n+1)= nck(n,m-1)*l^(n-m+1)*(1-l)^(m-1);
    end
end