function  L = loss(l, N)
% Creates a loss matrix, L, from a loss parameter and photon number N
% Loss matrix is now (N+1)x(N+1)
% Creator: Andy Worsley

% DEPENDENCIES
% Function 'nck.m' must be available - computes binomial coefficients
% without displaying errors

% INPUT
% l - loss parameter (1 - efficiency of detector)
% N - maximum number of photons to consider

% OUTPUT 
% L - Loss matrix

% -----
% START
% -----

n = 0; % Number of photons incident
m = 0; % m = k+1, where k = number of photons detected

L = zeros(N+1,N+1);

L(1,1)=1;
for n = 1:N
    for m = 1:(n+1)
        L(m,n+1)= nck(n,m-1)*l^(n-m+1)*(1-l)^(m-1);
    end
end