% Creator: Andy Worsley

% Finds the input state for a given set of click statistics and LOOPY
% losses.

% I have not found a least squares algorithm for an equation of the form
% P = Q[rho]R, where all the variables are matrices, so the first part of 
% the program reduces the problem to a standard p = Mx form for a least 
% squares fit, where the components of x are the diagonals of [rho].

% Function "Loss" must be available - creates loss matrices
% Function "nck" must be available - nchoosek error suppressed for high
% coefficients
% P_kn1 and P_kn2 must be created, and click must be available

% P_kn1: convolution matrix for LOOPY 1
% P_kn2: convolution matrix for LOOPY 2
% click: click counts


% START

% ---- INITIALISE AND PREALLOCATE VARIABLES ----
N = 8;

n = 0; % Counting Variables
m = 0;
prow = 0;
qrow = 1;
qcol = 1;
rrow = 1;
rcol = 1;
mcol = 1;
mrow = 1;
xrow = 0;
loopnum = 0;

%l1 = 0.9498; % Loss in 7-Feb-08 LOOPY 1
%l2 = 0.9540; % Loss in 7-Feb-08 LOOPY 2
%l1 = 0.8270; % Loss in 28-Feb-08 LOOPY 1
%l2 = 0.8705; % Loss in 28-Feb-08 LOOPY 2
%l1 = 0.9012; % Loss parameter in 5-Mar-08 LOOPY 1 using Magnification
%l2 = 0.9208; % Loss parameter in 5-Mar-08 LOOPY 2 using Magnification
%l1 = 0.8948; % Lowest loss parameter in 5-Mar-08
%l2 = 0.9145; % Lowest loss parameter in 5-Mar-08

l1 = 0.8978 + 0.004; % Loss parameter in 5-Mar-08 LOOPY 1 using Newton (+ error)
l2 = 0.9172 + 0.004;% Loss parameter in 5-Mar-08 LOOPY 2 using Newton (+ error)

%l1 = 0.8760; % Loss parameter in 5-Mar-08 LOOPY 1 optimising on all data runs at once
%l2 = 0.8971; % Loss parameter in 5-Mar-08 LOOPY 2 optimising on all data runs at once

L1 = zeros(N+1,N+1); % Loss matrix in LOOPY 1
L2 = zeros(N+1,N+1); % Loss matrix in LOOPY 2

Q = zeros(81,N+1); 
R = zeros(81,N+1);

p = zeros(81,1);
M = zeros(81,N+1);

flag = 0;

% Find statistics from click counts
P = click/sum(sum(click));

% Create columns of vectors for storing test results
% resnorm = zeros(1,numlosses^2);

x_bestrn = zeros((N+1)^2,1);
rho = zeros(N+1,N+1);

% ---- END OF INITIALISATION ----

% Check P_kn1 and P_kn2 are the same size
if ((size(P_kn1,1)~=size(P_kn2,1)) + (size(P_kn1,2)~=size(P_kn2,2)))
    error('Convolution matrices must be the same size');
end

% Deconstruct P to make p, an 81x1 vector
prow = 1;
for n = 1:9
    for m = 1:9
        p(prow,1) = P(n,m);
        prow = prow + 1;
    end
end

% Create loss matrices
L1 = Loss(l1, N); % Create loss matrix
L2 = Loss(l2, N); % Create loss matrix
        
% Calculate working matrices - initially need P = Q*rho*R
Q = P_kn1(1:9,1:9)*L1;
R = L2'*P_kn2(1:9,1:9)';       
        
% Create 81x(N+1) M matrix - to convert to form p = Mx
qrow = 1;
qcol = 1;
rrow = 1;
rcol = 1;
mcol = 1;

mrow = 1;
for qrow = 1:(N+1)
    for rrow = 1:(N+1)
        mcol = 1;
        for qcol = 1:(N+1)
            for rcol = 1:(N+1)
                M(mrow,mcol) = Q(qrow,qcol)*R(rcol,rrow);
            mcol = mcol + 1;
            end
        end
    mrow = mrow + 1;
    end
end


% Solve least squares problem to find photon statistics

[x_bestrn(:,1),resnorm] = lsqlin(M,p,[],[],[],[],zeros(81,1),ones(81,1),[],optimset('TolFun', 1e-35, 'Display', 'Off'));
%[x_bestrn(:,1)] = lsqr(M,p,10e-25,10000);
%[x_bestrn(:,1)] = lsqnonneg(M,p);


% Use 81 x_bestrn elements to reconstruct input state rho, 9x9 matrix
xrow = 1;
for n = 1:(N+1)
    for m = 1:(N+1)
        rho(n,m) = x_bestrn(xrow,1);
        xrow = xrow + 1;   
    end
end