%  FITTOMULTITHERM
% Takes a state rho and then varies s in a multithermal distribution M with
% the same mean in order to minimise norm(diag(M) - diag(rho)).

% An initial guess is provided e.g. 1 mode.

% INPUT: rho
% OUTPUTS: s, mean, resnorm

% Max number of photons to consider i.e. size of state
N = size(rho,2) - 1; % Read highest considered photon pair number from rho size

% Distances in parameter space for calculating derivatives
h = 0.05;
gamma = 1000;

Imax = 500;
iter = 1;
d = zeros(1,Imax);

% Starting vector
s_0 = 1;

% Tolerance
tol = 1e-5;

s = zeros(1,Imax);
s(1) = s_0;

T = zeros(2,3);
D = [0,0];

mean = meanvar(rho);

for iter = 1:Imax
    
    % ---- Calculating the function points for derivatives ----
    T(1,1) = s(1,iter); % Load point
    T(1,2) = s(1,iter) + h; % Load single variable points
    T(1,3) = s(1,iter) - h;
    
    % ---- Calculating the function values ----
    for i=1:3
        M = multitherm(mean(1), T(1,i), N);
        T(2,i) = norm(diag(M) - diag(rho), 'fro');
    end
    
    % ---- Derivatives ----
    % 1st derivative
    D(1) = (T(2,2) - T(2,3))/(2*h);
    % 2nd derivative
    D(2) = (T(2,2) + T(2,3) - T(2,1))/(h^2);
    
    % ---- Moving to a new point in parameter space ----
    % Points are recorded in 3 x Imax matrix, X, where Imax = max number of iterations
    if iter < Imax
       d(1,iter) = D(1)/D(2);
       s(1,iter+1) = s(1,iter) - gamma*d(1,iter);
    end
    
end
 