function [ P ] = multitherm( nbar, s, N )
% MULTITHERM produces a diagonal multithermal distribution with average
% photon number nbar, s modes and for 0 to N photons

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% INPUTS
% nbar - the mean photon number for the state
% s - the number of modes
% N - the maximum photon number to be considered

% OUTPUT
% P - an (N+1)x(N+1) matrix for the multithermal state

num = (0:N);
p = zeros(N+1,N+1);
P = zeros(N+1,N+1);

for n=1:N+1
    p(n,n) = (gamma(num(n)+s)/(factorial(num(n))*gamma(s)))*(1 + (nbar/s))^(-s)*(1+(s/nbar))^(-num(n));
end

P = p;
