function [ out ] = meanvar( rho )
% MEANVAR computes the mean photon number, variance and Q parameter of a
% joint number state rho

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% INPUT - rho, the square matrix state to be analysed
% OUTPUT - out, a 1x3 vector containing the mean, variance and Q parameter,
% where Q is estimated from an approximation that applies when mean << 1.
 
numvals = (0:8);
out = [0,0,0];

mean = (1/2)*(numvals*sum(rho,1)' + numvals*sum(rho,2));
var = (1/2)*((numvals.*numvals)*sum(rho,1)' + (numvals.*numvals)*sum(rho,2)) - mean^2;
Qparm = (var - mean)/mean^2;

out(1,1) = mean;
out(1,2) = var;
out(1,3) = Qparm;

